@@ -60,6 +60,7 @@ static struct job2str {
   { SOLVER_FAVOR,          "favor" },
   { SOLVER_DISFAVOR,       "disfavor" },
   { SOLVER_BLACKLIST,      "blacklist" },
+  { SOLVER_EXCLUDEFROMWEAK,   "excludefromweak" },
   { 0, 0 }
 };
 
@@ -1473,15 +1474,16 @@ testcase_solverresult(Solver *solv, int resultflags)
       SolverRuleinfo rclass;
       Queue q;
       int i;
+      char *prefix;
 
       queue_init(&q);
       for (rid = 1; (rclass = solver_ruleclass(solv, rid)) != SOLVER_RULE_UNKNOWN; rid++)
 	{
-	  char *prefix = solv_dupjoin("rule ", testcase_rclass2str(rclass), " ");
-	  prefix = solv_dupappend(prefix, testcase_ruleid(solv, rid), 0);
 	  solver_ruleliterals(solv, rid, &q);
 	  if (rclass == SOLVER_RULE_FEATURE && q.count == 1 && q.elements[0] == -SYSTEMSOLVABLE)
 	    continue;
+	  prefix = solv_dupjoin("rule ", testcase_rclass2str(rclass), " ");
+	  prefix = solv_dupappend(prefix, testcase_ruleid(solv, rid), 0);
 	  for (i = 0; i < q.count; i++)
 	    {
 	      Id p = q.elements[i];
@@ -1711,7 +1713,7 @@ testcase_write_mangled(Solver *solv, const char *dir, int resultflags, const cha
       for (rid = 1; rid < pool->nrels; rid++)
 	{
 	  Reldep *rd = pool->rels + rid;
-	  if (rd->flags != REL_NAMESPACE || rd->name == NAMESPACE_OTHERPROVIDERS)
+	  if (rd->flags != REL_NAMESPACE || rd->name == NAMESPACE_OTHERPROVIDERS || rd->name == NAMESPACE_SPLITPROVIDES)
 	    continue;
 	  /* evaluate all namespace ids, skip empty results */
 	  d = pool_whatprovides(pool, MAKERELDEP(rid));
@@ -1809,49 +1811,63 @@ testcase_write_mangled(Solver *solv, const char *dir, int resultflags, const cha
   return 1;
 }
 
-int
-testcase_write(Solver *solv, const char *dir, int resultflags, const char *testcasename, const char *resultname)
+const char **
+testcase_mangle_repo_names(Pool *pool)
 {
-  Pool *pool = solv->pool;
-  int i, r, repoid;
-  int mangle = 1;
-  const char **orignames;
-
-  /* mangle repo names so that there are no conflicts */
-  orignames = solv_calloc(pool->nrepos, sizeof(char *));
-  for (repoid = 1; repoid < pool->nrepos; repoid++)
+  int i, repoid, mangle = 1;
+  Repo *repo;
+  const char **names = solv_calloc(pool->nrepos, sizeof(char *));
+  FOR_REPOS(repoid, repo)
     {
-      Repo *repo = pool_id2repo(pool, repoid);
-      char *buf = solv_malloc((repo->name ? strlen(repo->name) : 0) + 40);
-      char *mp;
-      orignames[repoid] = repo->name;
+      char *buf, *mp;
+      buf = solv_malloc((repo->name ? strlen(repo->name) : 0) + 40);
       if (!repo->name || !repo->name[0])
         sprintf(buf, "#%d", repoid);
       else
 	strcpy(buf, repo->name);
-      for (i = 0; buf[i]; i++)
-	if (buf[i] == ' ' || buf[i] == '\t' || buf[i] == '/')
-	  buf[i] = '_';
-      mp = buf + strlen(buf);
-      for (;;)
-	{
-	  for (i = 1; i < repoid; i++)
-	    if (!strcmp(buf, pool_id2repo(pool, i)->name))
-	      break;
-	  if (i == repoid)
-	    break;
+      for (mp = buf; *mp; mp++)
+	if (*mp == ' ' || *mp == '\t' || *mp == '/')
+	  *mp = '_';
+      for (i = 1; i < repoid; i++)
+        {
+	  if (!names[i] || strcmp(buf, names[i]) != 0)
+	    continue;
           sprintf(mp, "_%d", mangle++);
+	  i = 0;	/* restart conflict check */
 	}
-      repo->name = buf;
+      names[repoid] = buf;
     }
-  r = testcase_write_mangled(solv, dir, resultflags, testcasename, resultname);
-  for (repoid = 1; repoid < pool->nrepos; repoid++)
+  return names;
+}
+
+static void
+swap_repo_names(Pool *pool, const char **names)
+{
+  int repoid;
+  Repo *repo;
+  FOR_REPOS(repoid, repo)
     {
-      Repo *repo = pool_id2repo(pool, repoid);
-      solv_free((void *)repo->name);
-      repo->name = orignames[repoid];
+      const char *n = repo->name;
+      repo->name = names[repoid];
+      names[repoid] = n;
     }
-  solv_free(orignames);
+}
+
+int
+testcase_write(Solver *solv, const char *dir, int resultflags, const char *testcasename, const char *resultname)
+{
+  Pool *pool = solv->pool;
+  int r, repoid;
+  const char **names;
+
+  /* mangle repo names so that there are no conflicts */
+  names = testcase_mangle_repo_names(pool);
+  swap_repo_names(pool, names);
+  r = testcase_write_mangled(solv, dir, resultflags, testcasename, resultname);
+  swap_repo_names(pool, names);
+  for (repoid = 1; repoid < pool->nrepos; repoid++)
+    solv_free((void *)names[repoid]);
+  solv_free((void *)names);
   return r;
 }
 
@@ -1977,6 +1993,7 @@ testcase_read(Pool *pool, FILE *fp, const char *testcase, Queue *job, char **res
   Id *genid = 0;
   int ngenid = 0;
   Queue autoinstq;
+  int oldjobsize = job ? job->count : 0;
 
   if (resultp)
     *resultp = 0;
@@ -2051,6 +2068,21 @@ testcase_read(Pool *pool, FILE *fp, const char *testcase, Queue *job, char **res
 	  int prio, subprio;
 	  const char *rdata;
 
+	  if (pool->considered)
+	    {
+	      pool_error(pool, 0, "testcase_read: cannot add repos after packages were disabled");
+	      continue;
+	    }
+	  if (solv)
+	    {
+	      pool_error(pool, 0, "testcase_read: cannot add repos after the solver was created");
+	      continue;
+	    }
+	  if (job && job->count != oldjobsize)
+	    {
+	      pool_error(pool, 0, "testcase_read: cannot add repos after jobs have been created");
+	      continue;
+	    }
 	  prepared = 0;
           if (!poolflagsreset)
 	    {
@@ -2064,7 +2096,7 @@ testcase_read(Pool *pool, FILE *fp, const char *testcase, Queue *job, char **res
 	    }
           repo->priority = prio;
           repo->subpriority = subprio;
-	  if (strcmp(pieces[3], "empty") != 0)
+	  if (strcmp(pieces[3], "empty") != 0 && npieces > 4)
 	    {
 	      const char *repotype = pool_tmpjoin(pool, pieces[3], 0, 0);	/* gets overwritten in <inline> case */
 	      if (!strcmp(pieces[4], "<inline>"))
@@ -2111,6 +2143,11 @@ testcase_read(Pool *pool, FILE *fp, const char *testcase, Queue *job, char **res
 	  int i;
 
 	  /* must set the disttype before the arch */
+	  if (job && job->count != oldjobsize)
+	    {
+	      pool_error(pool, 0, "testcase_read: cannot change the system after jobs have been created");
+	      continue;
+	    }
 	  prepared = 0;
 	  if (strcmp(pieces[2], "*") != 0)
 	    {
